%{
typedef struct yy_buffer_state *YY_BUFFER_STATE;
extern YY_BUFFER_STATE yy_scan_string(const char * str);
extern void yy_delete_buffer(YY_BUFFER_STATE buffer);

#include "rparser.h"
#include "lextypes.h"
#include <cstring>

yys yylval;

static YY_BUFFER_STATE buffer;

%}

%%
[0-9]+(\.[0-9]*)?((E|e)(\+|\-)?[0-9]+)?   {
            yylval.number = std::strtod(yytext, 0);
            return Token::NUMBER;
        }
"x"     { return Token::X; }
"y"     { return Token::Y; }
"z"     { return Token::Z; }
"pi"|"PI" {
            yylval.number = 3.14159265358979323846;
            return Token::NUMBER;
        }
"e"|"E" {
            yylval.number = 2.7182818284590452354;
            return Token::NUMBER;
        }
[a-zA-Z][a-zA-Z0-9]* {
            yylval.name = std::string(yytext);
            return Token::NAME;
        }
"+"     { return Token::PLUS; }
"-"     { return Token::MINUS; }
"**"|"^" { return Token::POW; }
"*"     { return Token::MUL; }
"/"     { return Token::DIV; }
"("     { return Token::LPAR; }
")"     { return Token::RPAR; }
","     { return Token::CM; }
"$"     { return Token::ENDLINE; }
" "|\n|\r\n|\t  { }
.       { return Token::UNDEFINED; }

%%

#ifdef STANDALONE

int main() {
    RParser parser;
    while(true) {
        char line[256];
        printf("Enter a scan line:\n");
        (void) fgets(line, 250, stdin);

        int l = int(strlen(line));
        // Erase "\r\n" at the end of line
        if (l > 0 && (line[l-1] == '\n' || line[l-1] == '\r'))
             --l;
        if (l > 0 && (line[l-1] == '\n' || line[l-1] == '\r'))
             --l;
        line[l] = 0;
        if (l == 0)     // Empty line => finish
            break;

        parser.initialize(line);
        printf("---------------\n");
        for (size_t i = 0; i < parser.tokens.size(); ++i) {
            //... printf("token=%s\n", parser.rpn[i].tokenName());
            printf("%s\n", parser.tokens[i].toString().c_str());
        }
        printf("---------------\n");

        if (!parser.parse()) {
            printf("Parse error\n");
            continue;
        }
        printf("Parsing success!\n");
        printf("---------------\n");
        for (size_t i = 0; i < parser.rpn.size(); ++i) {
            //... printf("token=%s\n", parser.rpn[i].tokenName());
            printf("%s\n", parser.rpn[i].toString().c_str());
        }
        printf("---------------\n");
        
        double v = parser.evaluate();
        printf("= %g\n", v);

        // line[l] = '$'; line[l+1] = 0;
        // YY_BUFFER_STATE buffer = yy_scan_string(line);
        // ...
        // yy_delete_buffer(buffer);           
    }    
    return 0;
}

#endif

int yywrap() {
    return 1;
}                    

void setInputBuffer(const char *txt) {
    buffer = yy_scan_string(txt);
}

void releaseInputBuffer() {
    yy_delete_buffer(buffer); 
}
